#!python
from __future__ import division,print_function
import numpy as np
class AtomicClosure:
    '''Baseclass for all atomic closures
    
    Note
    ----
    Currently, this class doesn't do anything besides group all of the
    *atomic* closures under a single inheritance heirarchy. 
    '''
        
