#!python
from __future__ import division,print_function
import numpy as np
class Closure:
    '''Baseclass for all closures 
    
    .. note::
    
        Currently, this class doesn't do anything besides group all of the
        closures under a single inheritance heirarchy. 
    '''
        
