#!python
from __future__ import division,print_function
import numpy as np
class Omega(object):
    '''Baseclass for all *intra*-molecular correlation functions
    
    .. note::
    
        Currently, this class doesn't do anything besides group all of the
        *intra*-molecular correlation functions under a single inheritance
        heirarchy. This will change as needs arise.
    '''
        
