#  tests for pyprism-1.0.3-py310h2873277_1004 (this is a generated file);
print('===== testing package: pyprism-1.0.3-py310h2873277_1004 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import pyPRISM
pyPRISM.test()
#  --- run_test.py (end) ---

print('===== pyprism-1.0.3-py310h2873277_1004 OK =====');
print("import: 'pyPRISM'")
import pyPRISM

print("import: 'pyPRISM.core'")
import pyPRISM.core

print("import: 'pyPRISM.calculate'")
import pyPRISM.calculate

print("import: 'pyPRISM.closure'")
import pyPRISM.closure

print("import: 'pyPRISM.omega'")
import pyPRISM.omega

print("import: 'pyPRISM.util'")
import pyPRISM.util

print("import: 'pyPRISM.potential'")
import pyPRISM.potential

print("import: 'pyPRISM.trajectory'")
import pyPRISM.trajectory

