#!python
from __future__ import division,print_function
import numpy as np
class Potential(object):
    '''Baseclass for all intermolecular-pairwise potentials
    
    ..note:: 
    
        Currently, this class doesn't do anything besides group all of the
        potentials under a single inheritance heirarchy. 
    
    '''
        
