#!python

from __future__ import division,print_function
import numpy as np

class MolecularClosure:
    '''Baseclass for all *molecular* closures
    
    .. note::

        Currently, this class doesn't do anything besides group all of the
        *molecular* closures under a single inheritance heirarchy. 
    '''
        
